@@ -28,34 +28,71 @@ function(p = 0.75, Dataset, class, ...)
     stop(sQuote("p"), " must be greater or equal than 0.0 and less or equal than 1.0!", call. = FALSE)
   }
   
-  output <- new("RCLS.chunk")  
+  output <- new("RCLS.chunk")
   
-  output@levels <- levels(factor(Dataset[, class]))
+  names <- colnames(Dataset)
+  
+  Dataset[, class] <- as.factor(Dataset[, class])
+
+  output@levels <- levels(Dataset[, class])  
+  
+  levels <- output@levels
   
   Dataset <- base::split(Dataset, Dataset[, class])
 
   output@s <- length(Dataset)
 
+  r <- integer()
+  
   for (i in 1:output@s) {
     n <- nrow(Dataset[[i]]) 
 
     output@ntrain[i] <- as.integer(n * p)
 
-    sample <- sample.int(n = n, size = output@ntrain[i], ...)
+    if (output@ntrain[i] > 1) {
+      sample <- sample.int(n = n, size = output@ntrain[i])
 
-    output@train[[i]] <- Dataset[[i]][sample,]
+      output@train[[i]] <- data.frame(Dataset[[i]][sample,])
+      
+      output@test <- rbind(output@test, Dataset[[i]][-sample,])
+    }
+    else {
+      r <- c(r, i)
 
-    output@test <- rbind(output@test, Dataset[[i]][-sample,])
+      output@test <- rbind(output@test, Dataset[[i]])
+    }
   }
+  
+  if (length(r) > 0) {
+    output@s <- output@s - length(r)
 
-  output@ntest <- nrow(output@test)
+    output@levels <- output@levels[-r]
 
-  output@train <- lapply(output@train, function(x) data.frame(x[, -class]))
+    output@train <- output@train[-r]
 
-  output@Zt <- factor(output@test[, class])
+    output@ntrain <- output@ntrain[-r]
+    
+    levels <- c(levels[-r], levels[r])
+  }
+  
+  output@train <- lapply(output@train, function(x) {x <- data.frame(x[, -class]); colnames(x) <- names[-class]; x})
+  
+  output@Zt <- factor(output@test[, class], levels = levels)
 
   output@test <- data.frame(output@test[, -class])
-
+  
+  colnames(output@test) <- names[-class]
+  
+  output@ntest <- nrow(output@test)
+  
+  if (output@s < 1) {
+    stop(sQuote("ntrain"), " number of observations in at least one train dataset must be greater than 1!", call. = FALSE)
+  }
+  
+  if (output@ntest < 1) {
+    stop(sQuote("ntest"), " number of observations in test dataset must be greater than 0!", call. = FALSE)
+  }  
+ 
   rm(list = ls()[!(ls() %in% c("output"))]) 
 
   invisible(output)
@@ -113,31 +150,66 @@ function(p = list(), Dataset, class, ...)
   
   if (!(p$test %in% type)) {
     stop(sQuote("p$test"), " should be one of ", paste(dQuote(type), collapse = ", "), "!", call. = FALSE)
-  }                  
-                 
+  }
+  
+  names <- colnames(Dataset)
+
+  Dataset[, class] <- as.factor(Dataset[, class])
+
+  output@levels <- levels(Dataset[, class])
+
   output@test <- subset(Dataset, subset = Dataset[, p$type] == p$test)
   
   output@ntest <- nrow(output@test)
+
+  levels <- output@levels
+
+  Dataset <- subset(Dataset, subset = Dataset[, p$type] == p$train)
   
-  output@Zt <- factor(output@test[, class])
+  Dataset <- base::split(Dataset, Dataset[, class])
+
+  output@s <- length(Dataset)
+
+  r <- integer()
+
+  for (i in 1:output@s) {
+    output@ntrain[i] <- nrow(Dataset[[i]]) 
+
+    if (output@ntrain[i] > 1) {
+      output@train[[i]] <- Dataset[[i]]
+    }
+    else {
+      r <- c(r, i)
+    }
+  }
+  
+  if (length(r) > 0) {
+    output@s <- output@s - length(r)
+
+    output@levels <- output@levels[-r]
+
+    output@train <- output@train[-r]
+
+    output@ntrain <- output@ntrain[-r]
+
+    levels <- c(levels[-r], levels[r])
+  }
+
+  output@train <- lapply(output@train, function(x) {x <- data.frame(x[, c(-class, -p$type)]); colnames(x) <- names[c(-class, -p$type)]; x})  
+
+  output@Zt <- factor(output@test[, class], levels = levels)
 
   output@test <- data.frame(output@test[, c(-class, -p$type)])
   
-  Dataset <- subset(Dataset, subset = Dataset[, p$type] == p$train)
+  colnames(output@test) <- names[c(-class, -p$type)]
   
-  output@levels <- levels(factor(Dataset[, class]))
-  
-  Dataset <- base::split(Dataset, Dataset[, class])
-  
-  output@s <- length(Dataset)
-  
-  for (i in 1:output@s) {
-    output@ntrain[i] <- nrow(Dataset[[i]])
-    
-    output@train[[i]] <- Dataset[[i]]
+  if (output@s < 1) {
+    stop(sQuote("ntrain"), " number of observations in at least one train dataset must be greater than 1!", call. = FALSE)
   }
   
-  output@train <- lapply(output@train, function(x) data.frame(x[, c(-class, -p$type)]))  
+  if (output@ntest < 1) {
+    stop(sQuote("ntest"), " number of observations in test dataset must be greater than 0!", call. = FALSE)
+  }   
 
   rm(list = ls()[!(ls() %in% c("output"))]) 
 
